 /* -*- mode: C++; compile-command: "flex -Pgiac_yy -olex.yy.c input_lexer.ll && mv -f lex.yy.c input_lexer.cc && g++ -g -I.. -c input_lexer.cc" -*- */
/** @file input_lexer.ll
 *
 *  Lexical analyzer definition for reading expressions.
 *  Note Maple input should be processed replacing # with // and { } for set
 *  This file must be processed with flex. */

/*
 *  Early version modified from GiNaC by B. Parisse (C) 2001
 *  GiNaC Copyright (C) 1999-2000 Johannes Gutenberg University Mainz, Germany
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


/*
 * The lexer will first check for static patterns and strings (defined below)
 * If a match is not found, it calls find_or_make_symbol
 * This function looks first if the string should be translated
 * (e.g. add a prefix from the export table)
 * then look in lexer_functions for a match, then look in sym_tab
 * if not found in sym_tab, a new identificateur is created & added in sym_tab
 * Functions in lexer_functions are added during the construction
 * of the corresponding unary_functions using lexer_functions_register
 */


/*
 *  Definitions
 */

%{
#include "first.h"
#include <iostream>
#include <stdexcept>

#include "input_lexer.h"
#include "gen.h"
#include "identificateur.h"
#include "usual.h"
#include "derive.h"
#include "series.h"
#include "intg.h"
#include "sym2poly.h"
#include "moyal.h"
#include "subst.h"
#include "vecteur.h"
#include "modpoly.h"
#include "lin.h"
#include "solve.h"
#include "ifactor.h"
#include "alg_ext.h"
#include "gauss.h"
#include "isom.h"
#include "plot.h"
#include "prog.h"
#include "rpn.h"
#include "ezgcd.h"
#include "tex.h"
#include "risch.h"
    
  using namespace std;
#ifndef NO_NAMESPACE_GIAC
  using namespace giac;
  namespace giac {
#endif // ndef NO_NAMESPACE_GIAC
    
#include "input_parser.h"
    sym_tab syms,lexer_functions;
    std::map<std::string,std::vector<std::string> > lexer_translator;
    // lexer_translator will be updated when export/with is called
    // To each string (w/o ::) in a given library, 
    // If it exists, we push_back the full string (with ::)
    // If not we create a vector with the full string
    // If a library is unexported we remove the corresponding entry in the 
    // vector and remove the entry if the vector is empty
    std::map<std::string,std::vector<std::string> > library_functions;
    // First string is the library name, second is the vector of function names
    vector<user_function> registered_lexer_functions; // User defined relations
    string parse_filename="// ";
    extern int maple_mode;
    int index_status=0; // 0 if [ -> T_VECT_DISPATCH, 1 if [ -> T_INDEX_BEGIN
    int opened_quote=0; // 1 if we are inside a quote
    int in_rpn=0; // 1 inside RPN expression
    int lexer_line_number=1;
    bool spread_formula=false; // beginning = and meaning of :
    bool initialisation_done=false;
    string comment_s;

    bool has_special_syntax(const string & s){
      sym_tab::const_iterator i = lexer_functions.find(s);
      if (i==lexer_functions.end())
	return false;
      return (i->second.subtype!=T_UNARY_OP);
    }
    
    bool lexer_functions_register(const unary_function_ptr & u,const string & s,int parser_token){
      sym_tab::const_iterator i = lexer_functions.find(s);
      if (i!=lexer_functions.end())
	return false;
      registered_lexer_functions.push_back(user_function(s,parser_token));
      lexer_functions[s] = gen(u);
      if (parser_token==1)
	lexer_functions[s].subtype=T_UNARY_OP;
      else
	lexer_functions[s].subtype=parser_token;
      // If s is a library function name (with ::), update the library
      int ss=s.size(),j=0;
      for (;j<ss-1;++j){
	if (s[j]==':' && s[j+1]==':')
	  break;
      }
      if (j<ss-1){
	string libname=s.substr(0,j);
	string funcname=s.substr(j+2,ss-j-2);
	std::map<std::string,std::vector<string> >::iterator it=library_functions.find(libname);
	if (it!=library_functions.end())
	  it->second.push_back(funcname);
	else
	  library_functions[libname]=vector<string>(1,funcname);
      }
      return true;
    }

    bool lexer_function_remove(const vector<user_function> & v){
      vector<user_function>::const_iterator it=v.begin(),itend=v.end();
      sym_tab::const_iterator i,iend;
      bool ok=true;
      for (;it!=itend;++it){
	i = lexer_functions.find(it->s);
	iend=lexer_functions.end();
	if (i==iend)
	  ok=false;
	else
	  lexer_functions.erase(it->s);
      }
      return ok;
    }

    int find_or_make_symbol(const string & s,gen & res){
      if (s.size()==1){
	switch (s[0]){
	case '+':
	  res=at_plus;
	  return T_UNARY_OP;
	case '-':
	  res=at_neg;
	  return T_UNARY_OP;
	case '*':
	  res=at_prod;
	  return T_UNARY_OP;
	case '/':
	  res=at_division;
	  return T_UNARY_OP;
	case '^':
	  res=at_pow;
	  return T_UNARY_OP;
	}
      }
      string ts(s);
      std::map<std::string,std::vector<string> >::const_iterator j=lexer_translator.find(s);
      if (j!=lexer_translator.end() && !j->second.empty())
	ts=j->second.back();
      sym_tab::const_iterator i = lexer_functions.find(ts);
      if (i!=lexer_functions.end()){
	if (i->second.subtype==T_TO)
	  res=plus_one;
	else
	  res = i->second;
	res.subtype=1;
	index_status=0;
	return i->second.subtype ;
      }
      i = syms.find(s);
      if (i == syms.end()) {
	// std::cerr << "lexer new" << s << endl;
	res = *(new identificateur(s));
	syms[s] = res;
      } else {
	// std::cerr << "lexer" << s << endl;
	res = i->second;
      }
      return T_SYMBOL;
    }

#ifndef NO_NAMESPACE_GIAC
  } // namespace giac
#endif // ndef NO_NAMESPACE_GIAC
  

      // lex input function
  static int lexer_input(char *buf, int max_size);
#define YY_INPUT(buf, result, max_size) (result = lexer_input(buf, max_size))
%}

	/* Abbreviations */
D	[0-9]
E	[eE][eE]?[-+]?{D}+
A	[a-zA-Z~]
AN	[0-9a-zA-Z_.~] 
        /* FIXME is . allowed inside alphanumeric */

%x comment
%x comment_hash
%x str
%x backquote
/*
 *  Lexical rules
 */

%%

[ 
\t\\]+			/* skip whitespace */
\n                ++lexer_line_number; //cerr << "Scanning line " << lexer_line_number << endl;
  /* Strings */
  /* \"[^\"]*\"        giac_yylval = string2gen( giac_yytext); return T_STRING; */
\"                BEGIN(str); comment_s="";
<str>\"\"         comment_s +='"';
<str>\"        {  index_status=1; BEGIN(INITIAL); 
                  giac_yylval=string2gen(comment_s,false); 
                  return T_STRING; }
<str>\n        comment_s+='\n'; ++lexer_line_number;
<str>\\[0-7]{1,3} {
                   /* octal escape sequence */
                   int result;
                   (void) sscanf( yytext + 1, "%o", &result );
                   comment_s += char(result & 0xff);
                   }
<str>\\[0-9]+      {
                   /* generate error - bad escape sequence; something
                    * like '\48' or '\0777777'
                    */
                   }
<str>\\n  comment_s += '\n';
<str>\\t  comment_s += '\t';
<str>\\r  comment_s += '\r';
<str>\\b  comment_s += '\b';
<str>\\f  comment_s += '\f';
<str>\\(.|\n)  comment_s += yytext[1];
<str>[^\\\n\"]+ comment_s += yytext;
`                  if (rpn_mode){ index_status=0; return T_ACCENTGRAVE; } else { BEGIN(backquote); comment_s=""; }
<backquote>\n      comment_s+='\n'; ++lexer_line_number;
<backquote>[^\n`]+       comment_s += yytext;
<backquote>`       {  index_status=1; BEGIN(INITIAL); 
     return find_or_make_symbol(comment_s,giac_yylval); }

"://"[^\n]*\n      index_status=0; ++lexer_line_number;
"//"[^\n]*\n      index_status=0; ++lexer_line_number;/* giac_yylval = string2gen('"'+string(giac_yytext).substr(2,string(giac_yytext).size()-3)+'"');   return T_COMMENT; */
"/*"              BEGIN(comment); comment_s="";

<comment>[^*\n]*        comment_s+=giac_yytext; /* eat anything that's not a '*' */
<comment>"*"+[^*/\n]*   comment_s+=giac_yytext; /* eat up '*'s not followed by '/'s */
<comment>\n             comment_s += '\n'; ++lexer_line_number; cerr << "(Comment) scanning line " << lexer_line_number << endl;
<comment>"*"+"/"        BEGIN(INITIAL); index_status=0; /* giac_yylval = string2gen(comment_s,false); return T_COMMENT; */
"#++"[^*]*"++#"         index_status=0; /* giac_yylval = string2gen('"'+string(giac_yytext).substr(3,string(giac_yytext).size()-6)+'"'); return T_COMMENT; */
"#--"[^*]*"--#"         index_status=0; /* giac_yylval = string2gen('"'+string(giac_yytext).substr(3,string(giac_yytext).size()-6)+'"'); return T_COMMENT; */

"?"                     if (index_status) return T_INTERROGATION; else return T_HELP;
"_"                     return T_UNIT;
"'"                     if (opened_quote) { opened_quote=0; return T_QUOTE; } if (index_status && !in_rpn && maple_mode!= 1) return T_PRIME; opened_quote=1; return T_QUOTE;
";"			index_status=0; if (maple_mode==3) return TI_SEMI; giac_yylval=0; return T_SEMI;
""                  index_status=0; if (maple_mode==3) return T_SEMI; return TI_SEMI;
":"			if (spread_formula) return T_DEUXPOINTS; if ( maple_mode==3 ) { index_status=0; return TI_DEUXPOINTS; }  index_status=0; if (maple_mode>0) { giac_yylval=1; return T_SEMI; } else return T_DEUXPOINTS;
":;"                    giac_yylval=1; return T_SEMI;
"::"                    return T_DOUBLE_DEUX_POINTS;

			/* special values */
"i"			index_status=1; if (maple_mode > 0 ) { giac_yylval=i__IDNT_e; return T_SYMBOL; } else { giac_yylval = cst_i; return T_LITERAL;};
\xa1                     index_status=1; giac_yylval = cst_i; return T_LITERAL;
"I"                     index_status=1; if (maple_mode==0 || maple_mode==3 || rpn_mode) { giac_yylval=I__IDNT_e; return T_SYMBOL; } else { giac_yylval = cst_i; return T_LITERAL; };
"pi"			index_status=1; giac_yylval = cst_pi; return T_LITERAL;
"Pi"			index_status=1; giac_yylval = cst_pi; return T_LITERAL;
"PI"			index_status=1; giac_yylval = cst_pi; return T_LITERAL;
"infinity"		index_status=1; giac_yylval = unsigned_inf; return T_LITERAL;
"unsigned_inf"		index_status=1; giac_yylval = unsigned_inf; return T_LITERAL;
"plus_inf"		index_status=1; giac_yylval = plus_inf; return T_LITERAL;
"minus_inf"		index_status=1; giac_yylval = minus_inf; return T_LITERAL;
"undef"		        index_status=1; giac_yylval = undef; return T_LITERAL;
""                     return T_END_INPUT;

               /* integer values */
"DOM_int"                  index_status=0; giac_yylval = _INT_; giac_yylval.subtype=_INT_TYPE; return T_TYPE_ID;
"DOM_INT"               index_status=0; giac_yylval = _ZINT; giac_yylval.subtype=_INT_TYPE; return T_TYPE_ID;
"integer"               index_status=0; giac_yylval = _ZINT; giac_yylval.subtype=_INT_TYPE; return T_TYPE_ID;
"posint"               index_status=0; giac_yylval = _POSINT; giac_yylval.subtype=_INT_MAPLECONVERSION; return T_TYPE_ID;
"negint"               index_status=0; giac_yylval = _NEGINT; giac_yylval.subtype=_INT_MAPLECONVERSION; return T_TYPE_ID;
"nonposint"               index_status=0; giac_yylval = _NONPOSINT; giac_yylval.subtype=_INT_MAPLECONVERSION; return T_TYPE_ID;
"nonnegint"               index_status=0; giac_yylval = _NONNEGINT; giac_yylval.subtype=_INT_MAPLECONVERSION; return T_TYPE_ID;
"DOM_COMPLEX"               index_status=0; giac_yylval = _CPLX; giac_yylval.subtype=_INT_TYPE; return T_TYPE_ID;
"complex"               index_status=0; giac_yylval = _CPLX; giac_yylval.subtype=_INT_TYPE; return T_TYPE_ID;
"DOM_FLOAT"                index_status=0; giac_yylval = _DOUBLE_; giac_yylval.subtype=_INT_TYPE; return T_TYPE_ID;
"float"                index_status=0; giac_yylval = _DOUBLE_; giac_yylval.subtype=_INT_TYPE; return T_TYPE_ID;
"DOM_LIST"              index_status=0; giac_yylval = _VECT; giac_yylval.subtype=_INT_TYPE; return T_TYPE_ID;
"DOM_MATRIX"              index_status=0; giac_yylval = _VECT; giac_yylval.subtype=_INT_TYPE; return T_TYPE_ID;
"array"              index_status=0; giac_yylval = _VECT; giac_yylval.subtype=_INT_TYPE; return T_TYPE_ID;
"DOM_IDENT"            index_status=0; giac_yylval = _IDNT; giac_yylval.subtype=_INT_TYPE; return T_TYPE_ID;
"DOM_SYMBOLIC"              index_status=0; giac_yylval = _SYMB; giac_yylval.subtype=_INT_TYPE; return T_TYPE_ID;
"algebraic"              index_status=0; giac_yylval = _SYMB; giac_yylval.subtype=_INT_TYPE; return T_TYPE_ID;
"DOM_RAT"              index_status=0; giac_yylval = _FRAC; giac_yylval.subtype=_INT_TYPE; return T_TYPE_ID;
"rational"              index_status=0; giac_yylval = _FRAC; giac_yylval.subtype=_INT_TYPE; return T_TYPE_ID;
"DOM_STRING"                index_status=0; giac_yylval = _STRNG; giac_yylval.subtype=_INT_TYPE; return T_TYPE_ID;
"operator"              if (maple_mode==2){ giac_yylval = gen(at_user_operator,6); index_status=0; return T_UNARY_OP; }  index_status=0; giac_yylval = _FUNC; giac_yylval.subtype=_INT_TYPE; return T_TYPE_ID;
"newton_solver"         index_status=1; giac_yylval = _NEWTON_SOLVER; giac_yylval.subtype=_INT_SOLVER;return T_NUMBER;
"dnewton_solver"         index_status=1; giac_yylval = _DNEWTON_SOLVER; giac_yylval.subtype=_INT_SOLVER;return T_NUMBER;
"newtonj_solver"         index_status=1; giac_yylval = _NEWTONJ_SOLVER; giac_yylval.subtype=_INT_SOLVER;return T_NUMBER;
"hybridj_solver"         index_status=1; giac_yylval = _HYBRIDJ_SOLVER; giac_yylval.subtype=_INT_SOLVER;return T_NUMBER;
"hybridsj_solver"         index_status=1; giac_yylval = _HYBRIDSJ_SOLVER; giac_yylval.subtype=_INT_SOLVER;return T_NUMBER;
"hybrid_solver"         index_status=1; giac_yylval = _HYBRID_SOLVER; giac_yylval.subtype=_INT_SOLVER;return T_NUMBER;
"hybrids_solver"         index_status=1; giac_yylval = _HYBRIDS_SOLVER; giac_yylval.subtype=_INT_SOLVER;return T_NUMBER;
"secant_solver"         index_status=1; giac_yylval = _SECANT_SOLVER; giac_yylval.subtype=_INT_SOLVER;return T_NUMBER;
"steffenson_solver"         index_status=1; giac_yylval = _STEFFENSON_SOLVER; giac_yylval.subtype=_INT_SOLVER;return T_NUMBER;
"bisection_solver"         index_status=1; giac_yylval = _BISECTION_SOLVER; giac_yylval.subtype=_INT_SOLVER;return T_NUMBER;
"falsepos_solver"         index_status=1; giac_yylval = _FALSEPOS_SOLVER; giac_yylval.subtype=_INT_SOLVER;return T_NUMBER;
"brent_solver"         index_status=1; giac_yylval = _BRENT_SOLVER; giac_yylval.subtype=_INT_SOLVER;return T_NUMBER;
"golub_reinsch_decomp"         index_status=1; giac_yylval = _GOLUB_REINSCH_DECOMP; giac_yylval.subtype=_INT_SOLVER;return T_NUMBER;
"golub_reinsch_mod_decomp"         index_status=1; giac_yylval = _GOLUB_REINSCH_MOD_DECOMP; giac_yylval.subtype=_INT_SOLVER;return T_NUMBER;
"jacobi_decomp"         index_status=1; giac_yylval = _JACOBI_DECOMP; giac_yylval.subtype=_INT_SOLVER;return T_NUMBER;
"minor_det"         index_status=1; giac_yylval = _MINOR_DET; giac_yylval.subtype=_INT_SOLVER;return T_NUMBER;
"rational_det"         index_status=1; giac_yylval = _RATIONAL_DET; giac_yylval.subtype=_INT_SOLVER;return T_NUMBER;
"keep_pivot"         index_status=1; giac_yylval = _KEEP_PIVOT; giac_yylval.subtype=_INT_SOLVER;return T_NUMBER;
"trapeze"         index_status=1; giac_yylval = _TRAPEZE; giac_yylval.subtype=_INT_SOLVER;return T_NUMBER;
"point_milieu"         index_status=1; giac_yylval = _POINT_MILIEU; giac_yylval.subtype=_INT_SOLVER;return T_NUMBER;
"rectangle_droit"         index_status=1; giac_yylval = _RECTANGLE_DROIT; giac_yylval.subtype=_INT_SOLVER;return T_NUMBER;
"rectangle_gauche"         index_status=1; giac_yylval = _RECTANGLE_GAUCHE; giac_yylval.subtype=_INT_SOLVER;return T_NUMBER;
"rempli"         index_status=1; giac_yylval = _FILL_POLYGON ; giac_yylval.subtype=_INT_COLOR ;return T_NUMBER;
"line_width_1"         index_status=1; giac_yylval = 0 ; giac_yylval.subtype=_INT_COLOR ;return T_NUMBER;
"line_width_2"         index_status=1; giac_yylval = _LINE_WIDTH_2 ; giac_yylval.subtype=_INT_COLOR ;return T_NUMBER;
"line_width_3"         index_status=1; giac_yylval = _LINE_WIDTH_3 ; giac_yylval.subtype=_INT_COLOR ;return T_NUMBER;
"line_width_4"         index_status=1; giac_yylval = _LINE_WIDTH_4 ; giac_yylval.subtype=_INT_COLOR ;return T_NUMBER;
"line_width_5"         index_status=1; giac_yylval = _LINE_WIDTH_5 ; giac_yylval.subtype=_INT_COLOR ;return T_NUMBER;
"line_width_6"         index_status=1; giac_yylval = _LINE_WIDTH_6 ; giac_yylval.subtype=_INT_COLOR ;return T_NUMBER;
"line_width_7"         index_status=1; giac_yylval = _LINE_WIDTH_7 ; giac_yylval.subtype=_INT_COLOR ;return T_NUMBER;
"line_width_8"         index_status=1; giac_yylval = _LINE_WIDTH_8 ; giac_yylval.subtype=_INT_COLOR ;return T_NUMBER;
"point_width_1"         index_status=1; giac_yylval = 0 ; giac_yylval.subtype=_INT_COLOR ;return T_NUMBER;
"point_width_2"         index_status=1; giac_yylval = _POINT_WIDTH_2 ; giac_yylval.subtype=_INT_COLOR ;return T_NUMBER;
"point_width_3"         index_status=1; giac_yylval = _POINT_WIDTH_3 ; giac_yylval.subtype=_INT_COLOR ;return T_NUMBER;
"point_width_4"         index_status=1; giac_yylval = _POINT_WIDTH_4 ; giac_yylval.subtype=_INT_COLOR ;return T_NUMBER;
"point_width_5"         index_status=1; giac_yylval = _POINT_WIDTH_5 ; giac_yylval.subtype=_INT_COLOR ;return T_NUMBER;
"point_width_6"         index_status=1; giac_yylval = _POINT_WIDTH_6 ; giac_yylval.subtype=_INT_COLOR ;return T_NUMBER;
"point_width_7"         index_status=1; giac_yylval = _POINT_WIDTH_7 ; giac_yylval.subtype=_INT_COLOR ;return T_NUMBER;
"point_width_8"         index_status=1; giac_yylval = _POINT_WIDTH_8 ; giac_yylval.subtype=_INT_COLOR ;return T_NUMBER;
"hidden_name"         index_status=1; giac_yylval = _HIDDEN_NAME ; giac_yylval.subtype=_INT_COLOR ;return T_NUMBER;
"quadrant1"         index_status=1; giac_yylval = _QUADRANT1 ; giac_yylval.subtype=_INT_COLOR ;return T_NUMBER;
"quadrant2"         index_status=1; giac_yylval = _QUADRANT2 ; giac_yylval.subtype=_INT_COLOR ;return T_NUMBER;
"quadrant3"         index_status=1; giac_yylval = _QUADRANT3 ; giac_yylval.subtype=_INT_COLOR ;return T_NUMBER;
"quadrant4"         index_status=1; giac_yylval = _QUADRANT4 ; giac_yylval.subtype=_INT_COLOR ;return T_NUMBER;
"point_losange"         index_status=1; giac_yylval = _POINT_LOSANGE ; giac_yylval.subtype=_INT_COLOR ;return T_NUMBER;
"point_plus"         index_status=1; giac_yylval = _POINT_PLUS ; giac_yylval.subtype=_INT_COLOR ;return T_NUMBER;
"point_carre"         index_status=1; giac_yylval = _POINT_CARRE  ; giac_yylval.subtype=_INT_COLOR ;return T_NUMBER;
"point_croix"         index_status=1; giac_yylval = 0  ; giac_yylval.subtype=_INT_COLOR ;return T_NUMBER;
"point_triangle"         index_status=1; giac_yylval = _POINT_TRIANGLE ; giac_yylval.subtype=_INT_COLOR ;return T_NUMBER;
"point_etoile"         index_status=1; giac_yylval = _POINT_ETOILE ; giac_yylval.subtype=_INT_COLOR ;return T_NUMBER;
"point_point"         index_status=1; giac_yylval = _POINT_POINT ; giac_yylval.subtype=_INT_COLOR ;return T_NUMBER;
"black"         index_status=1; giac_yylval = _BLACK; giac_yylval.subtype=_INT_COLOR ;return T_NUMBER;
"gomme"         index_status=1; giac_yylval = _WHITE; /* was 49 */ giac_yylval.subtype=_INT_COLOR ;return T_NUMBER;
"noir"         index_status=1; giac_yylval = _BLACK; giac_yylval.subtype=_INT_COLOR ;return T_NUMBER;
"red"         index_status=1; giac_yylval = _RED; giac_yylval.subtype=_INT_COLOR ;return T_NUMBER;
"rouge"         index_status=1; giac_yylval = _RED; giac_yylval.subtype=_INT_COLOR ;return T_NUMBER;
"green"         index_status=1; giac_yylval = _GREEN; giac_yylval.subtype=_INT_COLOR ;return T_NUMBER;
"vert"         index_status=1; giac_yylval = _GREEN; giac_yylval.subtype=_INT_COLOR ;return T_NUMBER;
"yellow"         index_status=1; giac_yylval = _YELLOW; giac_yylval.subtype=_INT_COLOR ;return T_NUMBER;
"jaune"         index_status=1; giac_yylval = _YELLOW; giac_yylval.subtype=_INT_COLOR ;return T_NUMBER;
"blue"         index_status=1; giac_yylval = _BLUE; giac_yylval.subtype=_INT_COLOR ;return T_NUMBER;
"bleu"         index_status=1; giac_yylval = _BLUE; giac_yylval.subtype=_INT_COLOR ;return T_NUMBER;
"magenta"         index_status=1; giac_yylval = _MAGENTA; giac_yylval.subtype=_INT_COLOR ;return T_NUMBER;
"cyan"         index_status=1; giac_yylval = _CYAN; giac_yylval.subtype=_INT_COLOR ;return T_NUMBER;
"white"         index_status=1; giac_yylval = _WHITE; giac_yylval.subtype=_INT_COLOR ;return T_NUMBER;
"blanc"         index_status=1; giac_yylval = _WHITE; giac_yylval.subtype=_INT_COLOR ;return T_NUMBER;
"solid_line"         index_status=1; giac_yylval = 0; giac_yylval.subtype=_INT_COLOR ;return T_NUMBER;
"dash_line"         index_status=1; giac_yylval = _DASH_LINE; giac_yylval.subtype=_INT_COLOR ;return T_NUMBER;
"dot_line"         index_status=1; giac_yylval = _DOT_LINE; giac_yylval.subtype=_INT_COLOR ;return T_NUMBER;
"dashdot_line"         index_status=1; giac_yylval = _DASHDOT_LINE; giac_yylval.subtype=_INT_COLOR ;return T_NUMBER;
"dashdotdot_line"         index_status=1; giac_yylval = _DASHDOTDOT_LINE; giac_yylval.subtype=_INT_COLOR ;return T_NUMBER;
"cap_flat_line"         index_status=1; giac_yylval = _CAP_FLAT_LINE; giac_yylval.subtype=_INT_COLOR ;return T_NUMBER;
"cap_round_line"         index_status=1; giac_yylval = _CAP_ROUND_LINE; giac_yylval.subtype=_INT_COLOR ;return T_NUMBER;
"cap_square_line"         index_status=1; giac_yylval = _CAP_SQUARE_LINE; giac_yylval.subtype=_INT_COLOR ;return T_NUMBER;
"adaptive"         index_status=1; giac_yylval =_ADAPTIVE ; giac_yylval.subtype=_INT_PLOT ;return T_NUMBER;
"axes"         index_status=1; giac_yylval =_AXES ; giac_yylval.subtype=_INT_PLOT ;return T_NUMBER;
"frames"         index_status=1; giac_yylval =_FRAMES ; giac_yylval.subtype=_INT_PLOT ;return T_NUMBER;
"color"         index_status=1; giac_yylval = _COLOR ; giac_yylval.subtype=_INT_PLOT ;return T_NUMBER;
"nstep"         index_status=1; giac_yylval = _NSTEP ; giac_yylval.subtype=_INT_PLOT ;return T_NUMBER;
"tstep"         index_status=1; giac_yylval = _TSTEP ; giac_yylval.subtype=_INT_PLOT ;return T_NUMBER;
"ustep"         index_status=1; giac_yylval = _USTEP ; giac_yylval.subtype=_INT_PLOT ;return T_NUMBER;
"vstep"         index_status=1; giac_yylval = _VSTEP ; giac_yylval.subtype=_INT_PLOT ;return T_NUMBER;
"xstep"         index_status=1; giac_yylval = _XSTEP ; giac_yylval.subtype=_INT_PLOT ;return T_NUMBER;
"ystep"         index_status=1; giac_yylval = _YSTEP ; giac_yylval.subtype=_INT_PLOT ;return T_NUMBER;
"zstep"         index_status=1; giac_yylval = _ZSTEP ; giac_yylval.subtype=_INT_PLOT ;return T_NUMBER;
"filled"         index_status=1; giac_yylval = _FILLED; giac_yylval.subtype=_INT_PLOT ;return T_NUMBER;
"font"         index_status=1; giac_yylval =_FONT ; giac_yylval.subtype=_INT_PLOT ;return T_NUMBER;
"labels"         index_status=1; giac_yylval = _LABELS; giac_yylval.subtype=_INT_PLOT ;return T_NUMBER;
"legend"         index_status=1; giac_yylval = _LEGEND; giac_yylval.subtype=_INT_PLOT ;return T_NUMBER;
"linestyle"         index_status=1; giac_yylval = _LINESTYLE; giac_yylval.subtype=_INT_PLOT ;return T_NUMBER;
"resolution"         index_status=1; giac_yylval =_RESOLUTION ; giac_yylval.subtype=_INT_PLOT ;return T_NUMBER;
"sample"         index_status=1; giac_yylval = _SAMPLE; giac_yylval.subtype=_INT_PLOT ;return T_NUMBER;
"scaling"         index_status=1; giac_yylval = _SCALING; giac_yylval.subtype=_INT_PLOT ;return T_NUMBER;
"style"         index_status=1; giac_yylval = _STYLE; giac_yylval.subtype=_INT_PLOT ;return T_NUMBER;
"symbol"         index_status=0; giac_yylval = _SYMBOL; giac_yylval.subtype=_INT_PLOT ;return T_TYPE_ID;
"symbolsize"         index_status=1; giac_yylval =_SYMBOLSIZE ; giac_yylval.subtype=_INT_PLOT ;return T_NUMBER;
"thickness"         index_status=1; giac_yylval = _THICKNESS; giac_yylval.subtype=_INT_PLOT ;return T_NUMBER;
"epaisseur"         index_status=1; giac_yylval = _THICKNESS; giac_yylval.subtype=_INT_PLOT ;return T_NUMBER;
"title"         index_status=1; giac_yylval = _TITLE; giac_yylval.subtype=_INT_PLOT ;return T_NUMBER;
"titre"         index_status=1; giac_yylval = _TITLE; giac_yylval.subtype=_INT_PLOT ;return T_NUMBER;
"titlefont"         index_status=1; giac_yylval = _TITLEFONT; giac_yylval.subtype=_INT_PLOT ;return T_NUMBER;
"view"         index_status=1; giac_yylval = _VIEW; giac_yylval.subtype=_INT_PLOT ;return T_NUMBER;
"axesfont"         index_status=1; giac_yylval =_AXESFONT ; giac_yylval.subtype=_INT_PLOT ;return T_NUMBER;
"coords"         index_status=1; giac_yylval = _COORDS; giac_yylval.subtype=_INT_PLOT ;return T_NUMBER;
"labelfont"         index_status=1; giac_yylval = _LABELFONT; giac_yylval.subtype=_INT_PLOT ;return T_NUMBER;
"labeldirections"         index_status=1; giac_yylval = _LABELDIRECTIONS; giac_yylval.subtype=_INT_PLOT ;return T_NUMBER;
"numpoints"         index_status=1; giac_yylval = _NUMPOINTS; giac_yylval.subtype=_INT_PLOT ;return T_NUMBER;
"tickmarks"         index_status=1; giac_yylval =_TICKMARKS ; giac_yylval.subtype=_INT_PLOT ;return T_NUMBER;
"xtickmarks"         index_status=1; giac_yylval =_XTICKMARKS ; giac_yylval.subtype=_INT_PLOT ;return T_NUMBER;
"set"         index_status=0; giac_yylval =_SET__VECT ; giac_yylval.subtype=_INT_MAPLECONVERSION ;return T_TYPE_ID;
"list"         if (maple_mode==3) { index_status=1; return find_or_make_symbol(giac_yytext,giac_yylval); } index_status=0; giac_yylval = _MAPLE_LIST ; giac_yylval.subtype=_INT_MAPLECONVERSION ;return T_TYPE_ID;
"vector"         index_status=0; giac_yylval = _MATRIX__VECT ; giac_yylval.subtype=_INT_MAPLECONVERSION ;return T_TYPE_ID;
"polynom"         index_status=0; giac_yylval =_POLY1__VECT ; giac_yylval.subtype=_INT_MAPLECONVERSION ;return T_TYPE_ID;
"trig"         index_status=1; giac_yylval =_TRIG ; giac_yylval.subtype=_INT_MAPLECONVERSION ;return T_NUMBER;
"expln"         index_status=1; giac_yylval =_EXPLN ; giac_yylval.subtype=_INT_MAPLECONVERSION ;return T_NUMBER;
"parfrac"         index_status=1; giac_yylval =_PARFRAC ; giac_yylval.subtype=_INT_MAPLECONVERSION ;return T_NUMBER;
"base"         index_status=1; giac_yylval =_BASE ; giac_yylval.subtype=_INT_MAPLECONVERSION ;return T_NUMBER;
"confrac"         index_status=1; giac_yylval =_CONFRAC ; giac_yylval.subtype=_INT_MAPLECONVERSION ;return T_NUMBER;
"fullparfrac"         index_status=1; giac_yylval =_FULLPARFRAC ; giac_yylval.subtype=_INT_MAPLECONVERSION ;return T_NUMBER;
"Delete"         index_status=1; giac_yylval =_DELETE_OPERATOR ; giac_yylval.subtype=_INT_MUPADOPERATOR ;return T_NUMBER;
"Prefix"         index_status=1; giac_yylval =_PREFIX_OPERATOR ; giac_yylval.subtype=_INT_MUPADOPERATOR ;return T_NUMBER;
"Postfix"         index_status=1; giac_yylval =_POSTFIX_OPERATOR ; giac_yylval.subtype=_INT_MUPADOPERATOR ;return T_NUMBER;
"Binary"         index_status=1; giac_yylval =_BINARY_OPERATOR ; giac_yylval.subtype=_INT_MUPADOPERATOR ;return T_NUMBER;
"Nary"         index_status=1; giac_yylval =_NARY_OPERATOR ; giac_yylval.subtype=_INT_MUPADOPERATOR ;return T_NUMBER;


    /* vector/polynom/matrice delimiters */
"seq["              giac_yylval = _SEQ__VECT; return T_VECT_DISPATCH;
"set["              giac_yylval = _SET__VECT; return T_VECT_DISPATCH;
"rpn_func["         giac_yylval = _RPN_FUNC__VECT; return T_VECT_DISPATCH;
"group["            giac_yylval = _GROUP__VECT; return T_VECT_DISPATCH;
"line["             giac_yylval = _LINE__VECT; return T_VECT_DISPATCH;
"vector["           giac_yylval = _VECTOR__VECT; return T_VECT_DISPATCH;
"matrix["           giac_yylval = _MATRIX__VECT; return T_VECT_DISPATCH;
"pnt["              giac_yylval = _PNT__VECT; return T_VECT_DISPATCH;
"curve["            giac_yylval = _CURVE__VECT; return T_VECT_DISPATCH;
"halfline["         giac_yylval = _HALFLINE__VECT; return T_VECT_DISPATCH;
"poly1["            giac_yylval = _POLY1__VECT; return T_VECT_DISPATCH;
"assume["           giac_yylval = _ASSUME__VECT; return T_VECT_DISPATCH;
"spreadsheet["      giac_yylval = _SPREAD__VECT; return T_VECT_DISPATCH;
"folder["      giac_yylval = _FOLDER__VECT; return T_VECT_DISPATCH;
"<"                     index_status=0; giac_yylval=gen(at_inferieur_strict,2);  return T_TEST_EQUAL;
">"                     index_status=0; giac_yylval=gen(at_superieur_strict,2); return T_TEST_EQUAL;
","                     index_status=0; return T_VIRGULE;
",,"                     index_status=0; return T_VIRGULE;
"("                     index_status=0; return T_BEGIN_PAR;
")"                     index_status=1; return T_END_PAR;
\[			if (index_status) { index_status=0; return T_INDEX_BEGIN; } else { giac_yylval = 0; return T_VECT_DISPATCH; } ;
\]			index_status=1; return T_VECT_END;
"%["                    index_status=0; return T_POLY1_BEGIN;
"%]"                    index_status=1; return T_POLY1_END;
"%%["                   index_status=0; return T_MATRICE_BEGIN;
"%%]"                   index_status=1; return T_MATRICE_END;
"%%%["                  index_status=0; return T_ASSUME_BEGIN;
"%%%]"                  index_status=1; return T_ASSUME_END;
    /* geometric delimiters */
"%("                    index_status=0; return T_GROUPE_BEGIN;
"%)"                    index_status=1; return T_GROUPE_END;
"%%("                   index_status=0; return T_LINE_BEGIN;
"%%)"                   index_status=1; return T_LINE_END;
"%%%("                  index_status=0; return T_VECTOR_BEGIN;
"%%%)"                  index_status=1; return T_VECTOR_END;
"%%%%("                 index_status=0; return T_CURVE_BEGIN;
"%%%%)"                 index_status=1; return T_CURVE_END;
    /* gen delimiters */
"{"                     index_status=0; if (rpn_mode) { giac_yylval=0; return T_VECT_DISPATCH; } if (maple_mode==3) return T_VECT_BEGIN; if (maple_mode > 0 ) return T_SET_BEGIN; else return T_BLOC_BEGIN;
"}"                     index_status=1; if (rpn_mode) return T_VECT_END; if (maple_mode==3) return T_VECT_END; if (maple_mode > 0) return T_SET_END; else return T_BLOC_END;
"%{"                    index_status=0; return T_SET_BEGIN;
"%}"                    index_status=1; return T_SET_END;
"%%{"                   index_status=0; return T_ROOTOF_BEGIN;
"%%}"                   index_status=1; return T_ROOTOF_END;
"%%%{"                  index_status=0; return T_SPOLY1_BEGIN;
"%%%}"                  index_status=1; return T_SPOLY1_END;
"<<"                    index_status=0; ++in_rpn; return T_RPN_BEGIN;
">>"                    index_status=0; --in_rpn; return T_RPN_END;

    /* Maple libraries names */
"linalg"                index_status=1; giac_yylval=_LINALG; giac_yylval.subtype=_INT_MAPLELIB; return T_MAPLELIB;
"numtheory"                index_status=1; giac_yylval=_NUMTHEORY; giac_yylval.subtype=_INT_MAPLELIB; return T_MAPLELIB;
"groebner"                index_status=1; giac_yylval=_GROEBNER; giac_yylval.subtype=_INT_MAPLELIB; return T_MAPLELIB;

    /* binary operators */
"->"                    index_status=0; return T_MAPSTO;
"-<"                    giac_yylval = gen(at_couleur,2); index_status=0; return T_INTERVAL;
"=="			index_status=0; giac_yylval=gen(at_same,2); return T_TEST_EQUAL;
"'=='"                  index_status=0; giac_yylval=gen(at_same,2); return T_QUOTED_BINARY;
"_equal"                  index_status=0; giac_yylval=gen(at_same,2); return T_QUOTED_BINARY;
"!="			index_status=0; giac_yylval=gen(at_different,2); return T_TEST_EQUAL;
"'!='"                  index_status=0; giac_yylval=gen(at_different,2); return T_QUOTED_BINARY;
"<>"			index_status=0; giac_yylval=gen(at_different,2); return T_TEST_EQUAL;
"'<>'"                  index_status=0; giac_yylval=gen(at_different,2); return T_QUOTED_BINARY;
"_unequal"                  index_status=0; giac_yylval=gen(at_different,2); return T_QUOTED_BINARY;
"'<='"                  index_status=0; giac_yylval=gen(at_inferieur_egal,2); return T_QUOTED_BINARY;
"_leequal"                  index_status=0; giac_yylval=gen(at_inferieur_egal,2); return T_QUOTED_BINARY;
"<="			index_status=0; giac_yylval=gen(at_inferieur_egal,2); return T_TEST_EQUAL;
"'<'"                  index_status=0; giac_yylval=gen(at_inferieur_strict,2); return T_QUOTED_BINARY;
"_less"                  index_status=0; giac_yylval=gen(at_inferieur_strict,2); return T_QUOTED_BINARY;
"'>'"                  index_status=0; giac_yylval=gen(at_superieur_strict,2); return T_QUOTED_BINARY;
">="			index_status=0; giac_yylval=gen(at_superieur_egal,2); return T_TEST_EQUAL;
"'>='"                  index_status=0; giac_yylval=gen(at_superieur_egal,2); return T_QUOTED_BINARY;
"="                     spread_formula=!index_status; index_status=0; giac_yylval=gen(at_equal,2); return T_EQUAL;
"'='"                  index_status=0; giac_yylval=gen(at_equal,2); return T_QUOTED_BINARY;
"$"                     index_status=0; giac_yylval=gen(at_dollar,2); if (maple_mode>0) return T_DOLLAR_MAPLE; else return T_DOLLAR;
"%$"                   index_status=0; giac_yylval=gen(at_dollar,2); return T_DOLLAR_MAPLE;
"'$'"                  index_status=0; giac_yylval=gen(at_dollar,2); return T_QUOTED_BINARY;
"_seqgen"                  index_status=0; giac_yylval=gen(at_dollar,2); return T_QUOTED_BINARY;
":="			index_status=0; giac_yylval=gen(at_sto,2); return T_AFFECT;
"':='"                  index_status=0; giac_yylval=gen(at_sto,2); return T_QUOTED_BINARY;
"_assign"                  index_status=0; giac_yylval=gen(at_sto,2); return T_QUOTED_BINARY;
"=>"                    index_status=0; giac_yylval=gen(at_sto,2); return TI_STO;
"=<"                    index_status=0; giac_yylval=gen(at_array_sto,2); return T_AFFECT;
"@"{D}+                   index_status=1; giac_yytext[0]='0'; giac_yylval = symb_double_deux_points(makevecteur(_IDNT_break,chartab2gen( giac_yytext))); return T_SYMBOL;
"@"                     if (maple_mode!=3) {index_status=0; giac_yylval=gen(at_compose,2); return T_COMPOSE; } BEGIN(comment_hash);
"@@"                     index_status=0; giac_yylval=gen(at_composepow,2); return T_POW;
"'@@'"                     index_status=0; giac_yylval=gen(at_composepow,2); return T_QUOTED_BINARY;
"_fnest"                     index_status=0; giac_yylval=gen(at_composepow,2); return T_QUOTED_BINARY;
"'@'"                  index_status=0; giac_yylval=gen(at_compose,2); return T_QUOTED_BINARY;
"_fconcat"                  index_status=0; giac_yylval=gen(at_compose,2); return T_QUOTED_BINARY;
"&&"                    index_status=0; giac_yylval=gen(at_and,2); return T_AND_OP;
"and"                   index_status=0; giac_yylval=gen(at_and,2); return T_AND_OP;
"AND"                   index_status=0; giac_yylval=gen(at_and,2); return T_AND_OP;
"'&&'"                  index_status=0; giac_yylval=gen(at_and,2); return T_QUOTED_BINARY;
"'and'"                 index_status=0; giac_yylval=gen(at_and,2); return T_QUOTED_BINARY;
"_and"                 index_status=0; giac_yylval=gen(at_and,2); return T_QUOTED_BINARY;
"|"                     index_status=0; giac_yylval=gen(at_tilocal,2); return T_PIPE;
"||"                    index_status=0; giac_yylval=gen(at_ou,2); return T_AND_OP;
"'||'"                  index_status=0; giac_yylval=gen(at_ou,2); return T_QUOTED_BINARY;
"or"                    index_status=0; giac_yylval=gen(at_ou,2); return T_AND_OP;
"'or'"                  index_status=0; giac_yylval=gen(at_ou,2); return T_QUOTED_BINARY;
"_or"                  index_status=0; giac_yylval=gen(at_ou,2); return T_QUOTED_BINARY;
"OR"                    index_status=0; giac_yylval=gen(at_ou,2); return T_AND_OP;
"xor"                    index_status=0; giac_yylval=gen(at_xor,2); return T_AND_OP;
"_xor"                  index_status=0; giac_yylval=gen(at_xor,2); return T_QUOTED_BINARY;
"'xor'"                  index_status=0; giac_yylval=gen(at_xor,2); return T_QUOTED_BINARY;
"XOR"                    index_status=0; giac_yylval=gen(at_xor,2); return T_AND_OP;
".."                    index_status=0; giac_yylval=gen(at_interval,2); return T_INTERVAL;
"'..'"                  index_status=0; giac_yylval=gen(at_interval,2); return T_QUOTED_BINARY;
"_range"                  index_status=0; giac_yylval=gen(at_interval,2); return T_QUOTED_BINARY;
"!"                     if (maple_mode || index_status) { giac_yylval=gen(at_factorial); return T_FACTORIAL; } else { index_status=0; giac_yylval=gen(at_not,1); return T_NOT; }

    /* standard functions */
"+"                     index_status=0; giac_yylval=gen(at_plus,2); return T_PLUS;
"++"                    index_status=0; giac_yylval=gen(at_increment,1); return T_FACTORIAL;
"+="                    index_status=0; giac_yylval=gen(at_increment,1); return T_PLUS;
"--"                    index_status=0; giac_yylval=gen(at_decrement,1); return T_FACTORIAL;
"-="                    index_status=0; giac_yylval=gen(at_decrement,1); return T_PLUS;
".+"                    index_status=0; giac_yylval=gen(at_plus,2); return T_PLUS;
"&"                     index_status=0; giac_yylval=gen(at_plus,2); return T_PLUS;
""                     index_status=0; giac_yylval=2; return T_SQ;
""                     index_status=0; giac_yylval=3; return T_SQ;
  /* "','"                   index_status=0; giac_yylval=gen(at_makevector,2); return T_QUOTED_BINARY; commented because of f('a','b') */
"'+'"                   index_status=0; giac_yylval=gen(at_plus,2); return T_QUOTED_BINARY;
"_plus"                   index_status=0; giac_yylval=gen(at_plus,2); return T_QUOTED_BINARY;
"-"                     index_status=0; giac_yylval=gen(at_binary_minus,2); return T_MOINS;
".-"                     index_status=0; giac_yylval=gen(at_binary_minus,2); return T_MOINS;
"'-'"                   index_status=0; giac_yylval=gen(at_binary_minus,2); return T_QUOTED_BINARY;
"_subtract"                   index_status=0; giac_yylval=gen(at_binary_minus,2); return T_QUOTED_BINARY;
"*"                     index_status=0; giac_yylval=gen(at_prod,2); return T_FOIS;
"*="                    index_status=0; giac_yylval=gen(at_multcrement,1); return T_FOIS;
"."                     index_status=0; giac_yylval=gen(at_prod,2); return T_FOIS;
"&*"                     index_status=0; giac_yylval=gen(at_ampersand_times,2); return T_FOIS;
"&^"                     index_status=0; giac_yylval=gen(at_quote_pow,2); return T_POW;
".*"                     index_status=0; giac_yylval=gen(at_pointprod,2); return T_FOIS;
"'*'"                   index_status=0; giac_yylval=gen(at_prod,2); return T_QUOTED_BINARY;
"_mult"                   index_status=0; giac_yylval=gen(at_prod,2); return T_QUOTED_BINARY;
"/"                     index_status=0; giac_yylval=gen(at_division,2); return T_DIV;
"/="                    index_status=0; giac_yylval=gen(at_divcrement,1); return T_DIV;
"./"                     index_status=0; giac_yylval=gen(at_pointdivision,2); return T_DIV;
"'/'"                   index_status=0; giac_yylval=gen(at_division,2); return T_QUOTED_BINARY;
"_divide"                   index_status=0; giac_yylval=gen(at_division,2); return T_QUOTED_BINARY;
"%"                     index_status=0; if (maple_mode==3) { giac_yylval=gen(at_pourcent); return T_FACTORIAL; } if (maple_mode==1) { giac_yylval=symbolic(at_ans,vecteur(0)); return T_NUMBER; }  if (maple_mode) giac_yylval=gen(at_irem,2); else giac_yylval=0; return T_MOD;
"'%'"                  index_status=0; giac_yylval=gen(at_irem,2); return T_QUOTED_BINARY;
"mod"                   index_status=0; if (maple_mode==3) { giac_yylval=gen(at_irem,2); return T_UNARY_OP; } else { if (maple_mode) giac_yylval=gen(at_irem,2); else giac_yylval=0; return T_MOD; }
"'mod'"                  index_status=0; giac_yylval=gen(at_irem,2); return T_QUOTED_BINARY;
"_mod"                  index_status=0; giac_yylval=gen(at_irem,2); return T_QUOTED_BINARY;
"MOD"                   index_status=0; return T_MOD;
"^"                     index_status=0; giac_yylval=gen(at_pow,2); return T_POW;
"**"                     index_status=0; giac_yylval=gen(at_pow,2); return T_POW;
".^"                     index_status=0; giac_yylval=gen(at_pointpow,2); return T_POW;
"'^'"                   index_status=0; giac_yylval=gen(at_pow,2); return T_QUOTED_BINARY;
"_power"                   index_status=0; giac_yylval=gen(at_pow,2); return T_QUOTED_BINARY;
"SWAP"                  giac_yylval = gen(at_SWAP,0); index_status=0; return T_RPN_OP;
"DROP"                  giac_yylval = gen(at_DROP,0); index_status=0; return T_RPN_OP;
"DUP"                   giac_yylval = gen(at_DUP,0); index_status=0; return T_RPN_OP;
"ROLL"                  giac_yylval = gen(at_ROLL,0); index_status=0; return T_RPN_OP;
"ROLLD"                 giac_yylval = gen(at_ROLLD,0); index_status=0; return T_RPN_OP;
"PICK"                  giac_yylval = gen(at_PICK,0); index_status=0; return T_RPN_OP;
"Digits"                  giac_yylval = gen(at_Digits,0); index_status=0; return T_DIGITS;
"DIGITS"                  giac_yylval = gen(at_Digits,0); index_status=0; return T_DIGITS;
"arccos"		giac_yylval = gen(at_acos,1); index_status=0; return T_UNARY_OP;
"randnorm"		giac_yylval = gen(at_randNorm,1); index_status=0; return T_UNARY_OP;
"arccosh"		giac_yylval = gen(at_acosh,1); index_status=0; return T_UNARY_OP;
"alg"                   giac_yylval = gen(at_alg,0); index_status=0; return T_UNARY_OP;
"angle_radian"		giac_yylval = gen(at_angle_radian,0); index_status=0; return T_DIGITS;
"approx_mode"		giac_yylval = gen(at_approx_mode,0); index_status=0; return T_DIGITS;
"args"			index_status=1; return T_ARGS;
"'args'"                index_status=0; giac_yylval=gen(at_args,0); return T_QUOTED_BINARY;
"posons"		giac_yylval = gen(at_assume,1); index_status=0; return T_UNARY_OP;
"arcsin"		giac_yylval = gen(at_asin,1); index_status=0; return T_UNARY_OP;
"arcsinh"		giac_yylval = gen(at_asinh,1); index_status=0; return T_UNARY_OP;
"at"			giac_yylval = gen(at_at,2); index_status=0; return T_UNARY_OP;
"arctan"		giac_yylval = gen(at_atan,1); index_status=0; return T_UNARY_OP;
"arctanh"		giac_yylval = gen(at_atanh,1); index_status=0; return T_UNARY_OP;
"backquote"		giac_yylval = gen(at_backquote,1); index_status=0; return T_UNARY_OP;
"bloc"  		giac_yylval = gen(at_bloc,1); index_status=0; return T_UNARY_OP;
"begin"                 index_status=0; return T_BLOC_BEGIN;
"BEGIN"                 index_status=0; return T_BLOC_BEGIN;
"break"  		index_status=0; giac_yylval=gen(at_break,0); return T_BREAK;
"BREAK"  		index_status=0; giac_yylval=gen(at_break,0); return T_BREAK;
"by"                    index_status=0; return T_BY;
"case"                  index_status=0; return T_CASE;
"CASE"                  index_status=0; return T_CASE;
"catch"                 index_status=0; return T_CATCH;
"continue"  		index_status=0; return T_CONTINUE;
"click"		        giac_yylval = gen(at_click,1); index_status=0; return T_UNARY_OP;
"comment"		giac_yylval = gen(at_comment,1); index_status=0; return T_UNARY_OP;
"complex_mode"		giac_yylval = gen(at_complex_mode,1); index_status=0; return T_DIGITS;
"complex_variables"	giac_yylval = gen(at_complex_variables,0); index_status=0; return T_DIGITS;
"CONT"		        giac_yylval = gen(at_cont,1); index_status=0; return T_UNARY_OP;
"couleur"               giac_yylval = gen(at_couleur,2); index_status=0; return T_UNARY_OP;
"affichage"             giac_yylval = gen(at_couleur,2); index_status=0; return T_UNARY_OP;
"DEBUG"		        giac_yylval = gen(at_debug,1); index_status=0; return T_UNARY_OP;
"default"               index_status=1; return T_DEFAULT;
"derive"		giac_yylval = gen(at_derive,2); index_status=0; return T_UNARY_OP;
"do"			index_status=0; giac_yylval=gen(at_for,4); return T_DO;
"DO"			index_status=0; giac_yylval=gen(at_for,4); return T_DO;
"dot"			giac_yylval = gen(at_prod,2); index_status=0; return T_UNARY_OP;
"D"  		if (maple_mode==1 || maple_mode==2) { giac_yylval = gen(at_function_diff,1); index_status=1; return T_UNARY_OP;} else { index_status=1; return find_or_make_symbol(giac_yytext,giac_yylval); }
"e"                     if (maple_mode==1 || maple_mode==2) { giac_yylval=e__IDNT_e; }else giac_yylval=symbolic(at_exp,1); return T_NUMBER;
"else"			index_status=0; giac_yylval=gen(at_ifte,3); return T_ELSE;
"ELSE"			index_status=0; giac_yylval=gen(at_ifte,3); return T_ELSE;
"elif"                  index_status=0; return T_ELIF;
"end"			index_status=0; return T_BLOC_END;
"END"			index_status=0; return T_BLOC_END;
"end_for"               index_status=0; return T_BLOC_END;
"end_if"                index_status=0; giac_yylval=gen(at_ifte,3); return T_BLOC_END;
"end if"                index_status=0; giac_yylval=gen(at_ifte,3); return T_BLOC_END;
"end do"                index_status=0; giac_yylval=gen(at_ifte,3); return T_BLOC_END;
"end proc"                index_status=0; giac_yylval=gen(at_ifte,3); return T_BLOC_END;
"end_while"             index_status=0; return T_BLOC_END;
"end_case"              index_status=0; return T_ENDCASE;
"end_proc"              index_status=0; return T_BLOC_END;
"epsilon"               giac_yylval = gen(at_epsilon,0); index_status=0; return T_DIGITS;
"equal"			giac_yylval = gen(at_equal,2); index_status=0; return T_UNARY_OP;
"error"		        index_status=0; giac_yylval=gen(at_throw,1); return T_RETURN;
"erase"                 giac_yylval = gen(at_erase,0); index_status=0; return T_UNARY_OP;
"ERROR"		        index_status=0; giac_yylval=gen(at_throw,1); return T_RETURN;
"esac"                  index_status=0; return T_ENDCASE;
"expand"                if (maple_mode==3) giac_yylval=gen(at_partfrac); else giac_yylval = gen(at_expand,1); index_status=0; return T_UNARY_OP;
"export"		giac_yylval = gen(at_insmod,1); index_status=0; return T_UNARY_OP;
"false"                  index_status=0; giac_yylval = zero; giac_yylval.subtype=_INT_BOOLEAN; return T_NUMBER;
"FALSE"                  index_status=0; giac_yylval = zero; giac_yylval.subtype=_INT_BOOLEAN; return T_NUMBER;
"fdistrib"		giac_yylval = gen(at_expand,1); index_status=0; return T_UNARY_OP;
"for"			index_status=0; giac_yylval=gen(at_for,4); return T_FOR;
"FOR"			index_status=0; giac_yylval=gen(at_for,4); return T_FOR;
"from"			index_status=0; return T_FROM;
"fi"                    index_status=0; return T_BLOC_END;
"global"                giac_yylval=1; index_status=0; return T_LOCAL;
"HALT"		        giac_yylval = gen(at_halt,1); index_status=0; return T_UNARY_OP;
"if"      		index_status=0; giac_yylval=gen(at_ifte,3); return T_IF;
"IF"      		index_status=0; giac_yylval=gen(at_ifte,3); return T_RPN_IF;
"ifte"      		index_status=0; giac_yylval=gen(at_ifte,3); return T_IFTE;
"IFTE"      		index_status=0; giac_yylval=gen(at_ifte,3); return T_IFTE;
"'ifte'"                index_status=0; giac_yylval=gen(at_ifte,3); return T_QUOTED_BINARY;
"'if'"                index_status=0; giac_yylval=gen(at_ifte,3); return T_QUOTED_BINARY;
"_if"                index_status=0; giac_yylval=gen(at_ifte,3); return T_QUOTED_BINARY;
"ifactors"      	if (maple_mode==1) giac_yylval = gen(at_maple_ifactors); else giac_yylval = gen(at_ifactors,1); index_status=0; return T_UNARY_OP;
"in"                    index_status=0; return T_IN;
"INTVX"			giac_yylval = gen(at_integrate,1); index_status=0; return T_UNARY_OP;
"inter"			giac_yylval = gen(at_inter,2); index_status=0; return T_UNARY_OP;
"intersect"             index_status=0; return T_INTERSECT;
"'intersect'"                  index_status=0; giac_yylval=gen(at_intersect,2); return T_QUOTED_BINARY;
"_intersect"                  index_status=0; giac_yylval=gen(at_intersect,2); return T_QUOTED_BINARY;
"isom"		        giac_yylval = gen(at_isom,1); index_status=0; return T_UNARY_OP;
"KILL"		        giac_yylval = gen(at_kill,1); index_status=0; return T_UNARY_OP;
"local"			giac_yylval=0; index_status=0; return T_LOCAL;
"localbloc"		index_status=0; return T_LOCALBLOC;
"LOCAL"			giac_yylval=0; index_status=0; return T_LOCAL;
"log"			giac_yylval = gen(at_ln,1); index_status=1; return T_UNARY_OP; /* index_status=1 to accept log[] for a basis log */
"mkisom"		giac_yylval = gen(at_mkisom,2); index_status=0; return T_UNARY_OP;
"minus"                 index_status=0; return T_MINUS;
"'minus'"                  index_status=0; giac_yylval=gen(at_minus,2); return T_QUOTED_BINARY;
"_minus"                  index_status=0; giac_yylval=gen(at_minus,2); return T_QUOTED_BINARY;
"next"		        index_status=0; return T_CONTINUE;
"NEXT"		        index_status=0; return T_CONTINUE;
"not"                 giac_yylval = gen(at_not,1); if (maple_mode) return T_NOT;  index_status=0; return T_UNARY_OP;
"'not'"                  index_status=0; giac_yylval=gen(at_not,1); return T_QUOTED_BINARY;
"_not"                  index_status=0; giac_yylval=gen(at_not,1); return T_QUOTED_BINARY;
"NULL"                  index_status=1; return T_NULL;
"normalf"		giac_yylval = gen(at_greduce,1); index_status=0; return T_UNARY_OP;
"od"                    index_status=0; return T_BLOC_END;
"of"			index_status=0; return T_OF;
"'of'"                  index_status=0; giac_yylval=gen(at_of,2); return T_QUOTED_BINARY;
"op"                    if (maple_mode==1) giac_yylval = gen(at_maple_op,1); else giac_yylval = gen(at_feuille,1); index_status=0; return T_UNARY_OP;
"option"                giac_yylval=2; index_status=0; return T_LOCAL;
"otherwise"             index_status=0; return T_DEFAULT;
"Psi"			giac_yylval = gen(at_psi,2); index_status=0; return T_UNARY_OP;
"pcoef"		        giac_yylval = gen(at_pcoeff,1); index_status=0; return T_UNARY_OP;
"plotfunc2d" 		giac_yylval = gen(at_funcplot,2); index_status=0; return T_UNARY_OP;
"user_operator" 		giac_yylval = gen(at_user_operator,6); index_status=0; return T_UNARY_OP;
"Ox" 		        giac_yylval = _droite(makevecteur(zero,plus_one),0); return T_EXPRESSION;
"Oy" 		        giac_yylval = _droite(makevecteur(zero,cst_i),0); return T_EXPRESSION;
"program"		index_status=0; return T_PROGRAM;
"proc"                  index_status=0; return T_PROC;
"purge"                 if (rpn_mode) {giac_yylval=gen(at_purge,0); index_status=0; return T_RPN_OP;} else {giac_yylval = gen(at_purge,1); index_status=0; return T_UNARY_OP;};
"unassign"                 if (rpn_mode) {giac_yylval=gen(at_purge,0); index_status=0; return T_RPN_OP;} else {giac_yylval = gen(at_purge,1); index_status=0; return T_UNARY_OP;};
"PURGE"                 if (rpn_mode) {giac_yylval=gen(at_purge,0); index_status=0; return T_RPN_OP;} else {giac_yylval = gen(at_purge,1); index_status=0; return T_UNARY_OP;};
"randseed"			giac_yylval = gen(at_srand,1); index_status=0; return T_RETURN;
"rcl"			giac_yylval = gen(at_RCL,1); index_status=0; return T_UNARY_OP;
"RCL"			giac_yylval = gen(at_RCL,1); index_status=0; return T_UNARY_OP;
"read"			giac_yylval = gen(at_read,1); index_status=0; return T_RETURN;
"repeat"                index_status=0; return T_REPEAT;
"REPEAT"                index_status=0; return T_REPEAT;
"return"		giac_yylval = gen(at_return,1) ; index_status=0; return T_RETURN;
"RETURN"		giac_yylval = gen(at_return,1) ; index_status=0; return T_RETURN;
"'return'"              giac_yylval = gen(at_return,1) ; index_status=0; return T_QUOTED_BINARY;
"risch"		giac_yylval = gen(at_risch,4); index_status=0; return T_UNARY_OP;
"root"                giac_yylval = gen(at_maple_root,1); index_status=1; return T_UNARY_OP;
"rpn"                   giac_yylval = gen(at_rpn,0); index_status=0; return T_UNARY_OP;
"same"                  giac_yylval = gen(at_same,1); index_status=0; return T_UNARY_OP;
"signal"                giac_yylval = gen(at_signal,1); index_status=0; return T_UNARY_OP;
"SST"		        giac_yylval = gen(at_sst,1); index_status=0; return T_UNARY_OP;
"SST_IN"		giac_yylval = gen(at_sst_in,1); index_status=0; return T_UNARY_OP;
"srand"			giac_yylval = gen(at_srand,1); index_status=0; return T_RETURN;
"START"                 index_status=0; return T_START;
"stack"                 index_status=0; return T_STACK;
"step"                  index_status=0; return T_BY;
"STEP"                  index_status=0; return T_BY;
"STO"			giac_yylval = gen(at_sto,0); index_status=0; return T_UNARY_OP;
"subs"			if (maple_mode==1) giac_yylval = gen(at_maple_subs,2); else giac_yylval = gen(at_subs,2); index_status=0; return T_UNARY_OP;
"subsop"		if (maple_mode==1) giac_yylval = gen(at_maple_subsop,2); else giac_yylval = gen(at_subsop,2); index_status=0; return T_UNARY_OP;
"switch"                index_status=0; return T_SWITCH;
"SWITCH"                index_status=0; return T_SWITCH;
"tanh"			giac_yylval = gen(at_tanh,1); index_status=0; return T_UNARY_OP;
"then"		        index_status=0; giac_yylval=gen(at_ifte,3); return T_THEN;
"THEN"		        index_status=0; giac_yylval=gen(at_ifte,3); return T_THEN;
"throw"		        index_status=0; giac_yylval=gen(at_throw,1); return T_RETURN;
"to"                    giac_yylval=1; return T_TO;
"TO"                    giac_yylval=1; return T_TO;
"downto"                giac_yylval=-1; return T_TO;
"DOWNTO"                giac_yylval=-1; return T_TO;
"true"                  index_status=0; giac_yylval = plus_one; giac_yylval.subtype=_INT_BOOLEAN; return T_NUMBER;
"TRUE"                  index_status=0; giac_yylval = plus_one; giac_yylval.subtype=_INT_BOOLEAN; return T_NUMBER;
"try"                   index_status=0; return T_TRY;
"try_catch"             index_status=0; return T_TRY_CATCH;
"union"                 index_status=0; return T_UNION;
"'union'"                  index_status=0; giac_yylval=gen(at_union,2); return T_QUOTED_BINARY;
"_union"                  index_status=0; giac_yylval=gen(at_union,2); return T_QUOTED_BINARY;
"until"                 index_status=0; return T_UNTIL;
"UNTIL"                 index_status=0; return T_UNTIL;
"virgule"               giac_yylval = gen(at_virgule,2); index_status=0; return T_UNARY_OP;
"VARS"                  giac_yylval = gen(at_VARS,0); index_status=0; return T_UNARY_OP;
"scientific_format"		        giac_yylval = gen(at_scientific_format,0); index_status=0; return T_DIGITS;
"while"                 index_status=0; giac_yylval=gen(at_for,4); if (maple_mode==3) return TI_WHILE; if (maple_mode!=0) return T_MUPMAP_WHILE; return T_WHILE;
"WHILE"                 index_status=0; giac_yylval=gen(at_for,4); return T_RPN_WHILE;
"Text"                  giac_yylval = gen(at_Text,1); index_status=0; return T_RETURN;
"DropDown"                  giac_yylval = gen(at_DropDown,1); index_status=0; return T_RETURN;
"Popup"                  giac_yylval = gen(at_Popup,1); index_status=0; return T_RETURN;
"Request"                  giac_yylval = gen(at_Request,1); index_status=0; return T_RETURN;
"Title"                  giac_yylval = gen(at_Title,1); index_status=0; return T_RETURN;
"Define"               giac_yylval=0; index_status=0; return TI_DEFINE;
":Prgm"                 giac_yylval=0; index_status=0; return TI_PRGM;
"Prgm"                 giac_yylval=0; index_status=0; return TI_PRGM;
"EndPrgm"              giac_yylval=0; index_status=0; return T_BLOC_END;
"Endprgm"              giac_yylval=0; index_status=0; return T_BLOC_END;
":Func"                 giac_yylval=0; index_status=0; return TI_PRGM;
"Func"                 giac_yylval=0; index_status=0; return TI_PRGM;
"func"                 giac_yylval=0; index_status=0; return TI_PRGM;
":func"                 giac_yylval=0; index_status=0; return TI_PRGM;
"EndFunc"              giac_yylval=0; index_status=0; return T_BLOC_END;
"endfunc"              giac_yylval=0; index_status=0; return T_BLOC_END;
"Endfunc"              giac_yylval=0; index_status=0; return T_BLOC_END;
"Local"                giac_yylval=0; index_status=0; return TI_LOCAL;
"If"      	       index_status=0; giac_yylval=gen(at_ifte,3); return T_IF;
"Then"                 index_status=0; giac_yylval=gen(at_ifte,3); return T_THEN;
"Else"                 index_status=0; giac_yylval=gen(at_ifte,3); return T_ELSE;
"EndIf"                index_status=0; giac_yylval=gen(at_ifte,3); return T_BLOC_END;
"Endif"                index_status=0; giac_yylval=gen(at_ifte,3); return T_BLOC_END;
"Return"	       giac_yylval = gen(at_return,1) ; index_status=0; return T_RETURN;
"ElseIf"               giac_yylval = gen(at_ifte,3) ;  index_status=0; return T_ELIF;
"Elseif"               giac_yylval = gen(at_ifte,3) ;  index_status=0; return T_ELIF;
"Exit"  	       index_status=0; giac_yylval=gen(at_breakpoint,0); return T_BREAK;
"Loop"                  index_status=0; giac_yylval=gen(at_for,0); return TI_LOOP;
"EndLoop"               index_status=0; giac_yylval=gen(at_for,0); return T_BLOC_END;
"Endloop"               index_status=0; giac_yylval=gen(at_for,0); return T_BLOC_END;
"For"                   index_status=0; giac_yylval=gen(at_for,0); return TI_FOR;
"EndFor"               index_status=0; giac_yylval=gen(at_for,0); return T_BLOC_END;
"Endfor"               index_status=0; giac_yylval=gen(at_for,0); return T_BLOC_END;
"While"                   index_status=0; giac_yylval=gen(at_for,0); return TI_WHILE;
"EndWhile"               index_status=0; giac_yylval=gen(at_for,0); return T_BLOC_END;
"Endwhile"               index_status=0; giac_yylval=gen(at_for,0); return T_BLOC_END;
"Cycle"               index_status=0; giac_yylval=gen(at_for,0); return T_CONTINUE;
"Try"           index_status=0; giac_yylval=gen(at_for,0); return TI_TRY;
"EndTry"           index_status=0; giac_yylval=gen(at_for,0); return T_BLOC_END;
"Endtry"           index_status=0; giac_yylval=gen(at_for,0); return T_BLOC_END;
"Disp"	       giac_yylval = gen(at_print,1) ; index_status=0; return T_RETURN;
"Pause"	       giac_yylval = gen(at_Pause,1) ; index_status=0; return T_RETURN;
"DelVar"       giac_yylval = gen(at_DelVar,1) ; index_status=0; return T_RETURN;
"label"	       giac_yylval = gen(at_label,1) ; index_status=0; return T_RETURN;
"Lbl"	       giac_yylval = gen(at_label,1) ; index_status=0; return T_RETURN;
"goto"	       giac_yylval = gen(at_goto,1) ; index_status=0; return T_RETURN;
"Goto"	       giac_yylval = gen(at_goto,1) ; index_status=0; return T_RETURN;
"Dialog"       giac_yylval = gen(at_Dialog,1) ; index_status=0; return TI_DIALOG; 
"EndDlog"    giac_yylval = gen(at_Dialog,1) ; index_status=0; return T_BLOC_END;
"Row"	       giac_yylval = gen(at_Row,0) ; index_status=0; return T_DIGITS;
"Col"	       giac_yylval = gen(at_Col,0) ; index_status=0; return T_DIGITS;
"threads"      giac_yylval = gen(at_threads,0) ; index_status=0; return T_DIGITS;
"_hbar_"        giac_yylval = cst_hbar; index_status=0; return T_SYMBOL;
"_c_"        giac_yylval = cst_clightspeed; index_status=0; return T_SYMBOL;
"_g_"        giac_yylval = cst_ga; index_status=0; return T_SYMBOL;
"_IO_" giac_yylval = cst_IO; index_status=0; return T_SYMBOL; 
"_epsilonox_" giac_yylval = cst_epsilonox; index_status=0; return T_SYMBOL; 
"_epsilonsi_" giac_yylval = cst_epsilonsi; index_status=0; return T_SYMBOL; 
"_qepsilon0_" giac_yylval = cst_qepsilon0; index_status=0; return T_SYMBOL; 
"_epsilon0q_" giac_yylval = cst_epsilon0q; index_status=0; return T_SYMBOL; 
"_kq_" giac_yylval = cst_kq; index_status=0; return T_SYMBOL; 
"_c3_" giac_yylval = cst_c3; index_status=0; return T_SYMBOL; 
"_lambdac_" giac_yylval = cst_lambdac; index_status=0; return T_SYMBOL; 
"_f0_" giac_yylval = cst_f0; index_status=0; return T_SYMBOL; 
"_lambda0_" giac_yylval = cst_lambda0; index_status=0; return T_SYMBOL; 
"_muN_" giac_yylval = cst_muN; index_status=0; return T_SYMBOL; 
"_muB_" giac_yylval = cst_muB; index_status=0; return T_SYMBOL; 
"_a0_" giac_yylval = cst_a0; index_status=0; return T_SYMBOL; 
"_Rinfinity_" giac_yylval = cst_Rinfinity; index_status=0; return T_SYMBOL; 
"_Faraday_" giac_yylval = cst_Faraday; index_status=0; return T_SYMBOL; 
"_phi_" giac_yylval = cst_phi; index_status=0; return T_SYMBOL; 
"_alpha_" giac_yylval = cst_alpha; index_status=0; return T_SYMBOL; 
"_mpme_" giac_yylval = cst_mpme; index_status=0; return T_SYMBOL; 
"_mp_" giac_yylval = cst_mp; index_status=0; return T_SYMBOL; 
"_qme_" giac_yylval = cst_qme; index_status=0; return T_SYMBOL; 
"_me_" giac_yylval = cst_me; index_status=0; return T_SYMBOL; 
"_qe_" giac_yylval = cst_qe; index_status=0; return T_SYMBOL; 
"_h_" giac_yylval = cst_hPlanck; index_status=0; return T_SYMBOL; 
"_G_" giac_yylval = cst_G; index_status=0; return T_SYMBOL; 
"_mu0_" giac_yylval = cst_mu0; index_status=0; return T_SYMBOL; 
"_epsilon0_" giac_yylval = cst_epsilon0; index_status=0; return T_SYMBOL; 
"_sigma_" giac_yylval = cst_sigma; index_status=0; return T_SYMBOL; 
"_StdP_" giac_yylval = cst_StdP; index_status=0; return T_SYMBOL; 
"_StdT_" giac_yylval = cst_StdT; index_status=0; return T_SYMBOL; 
"_R_" giac_yylval = cst_Rydberg; index_status=0; return T_SYMBOL; 
"_Vm_" giac_yylval = cst_Vm; index_status=0; return T_SYMBOL; 
"_k_" giac_yylval = cst_kBoltzmann; index_status=0; return T_SYMBOL; 
"_NA_" giac_yylval = cst_NA; index_status=0; return T_SYMBOL; 
"_in"        giac_yylval = symbolic(at_unit,makevecteur(1,_in_unit)); index_status=0; return T_SYMBOL;
 
			/* numbers */
{D}+			|
"#"[0-7]+"o"		|
"#"[0-1]+"b"		|
"#"[0-9a-fA-F]+"h"	|
"#o"[0-7]+		|
"#b"[0-1]+		|
"#x"[0-9a-fA-F]+	|
"0o"[0-7]+		|
"0b"[0-1]+		|
"0x"[0-9a-fA-F]+	|
{D}+"."{D}*({E})?	|
{D}*"."{D}+({E})?	|
{D}+{E}			{ 
  index_status=1;
  int l=strlen(giac_yytext);
  char ch,ch2;
  if (l>2 && giac_yytext[1]!='x' && (giac_yytext[l-1]=='o' || giac_yytext[l-1]=='b' || giac_yytext[l-1]=='h') ){
    char base=giac_yytext[l-1];
    for (int i=l-1;i>1;--i){
      giac_yytext[i]=giac_yytext[i-1];
    }
    if (base=='h')
      base='x';
    giac_yytext[1]=base;
  }
  else {
    for (l=0;(ch=*(giac_yytext+l));++l){
      if (ch=='x')
	break;
      if (ch=='e' || ch=='E'){
	if ( (ch2=*(giac_yytext+l+1)) && (ch2=='e' || ch2=='E')){
	  ++l;
	  for (;(ch=*(giac_yytext+l));++l)
	    *(giac_yytext+l-1)=ch;
	  *(giac_yytext+l-1)=0;
	  --l;
	  break;
	}
      }
    }
  }
  giac_yylval = chartab2gen(giac_yytext); return T_NUMBER; 
}

			/* symbols */
{A}{AN}*		{
  index_status=1; 
  return find_or_make_symbol(giac_yytext,giac_yylval);
}
"#"                     if (!maple_mode || maple_mode==3) { 
  // cerr << "hash" << endl;
  giac_yylval=gen(at_hash,1); return TI_HASH; 
} else BEGIN(comment_hash);
<comment_hash>[^*\n]*\n BEGIN(INITIAL); index_status=0; ++lexer_line_number;  /* comment_s=string(giac_yytext); giac_yylval=string2gen(comment_s.substr(0,comment_s.size()-1),false); return T_COMMENT; */
			/* everything else */
.			giac_yylval=string2gen(string(giac_yytext),false); return T_STRING;

%%


/*
 *  Routines
 */

// The string from which we will read
static std::string lexer_string;

// The current position within the string
static int curr_pos = 0;

// Input function that reads from string
static int lexer_input(char *buf, int max_size)
{
  int actual = lexer_string.length() - curr_pos;
  if (actual > max_size)
    actual = max_size;
  if (actual <= 0)
    return YY_NULL;
  lexer_string.copy(buf, actual, curr_pos);
  curr_pos += actual;
  return actual;
}

// EOF encountered, terminate the scanner
int giac_yywrap()
{
  return 1;
}

#ifndef NO_NAMESPACE_GIAC
namespace giac {
#endif // ndef NO_NAMESPACE_GIAC

        // Set the input string
  void set_lexer_string(const std::string &s_orig){
    string s(s_orig);
    bool instring=false;
    // stupid match of bracket then parenthesis
    int l=s.size(),nb=0,np=0;
    int i=0;
    for (;i<l;++i){
      if (s[i]==92){
	i += 2;
	if (i>=l)
	  break;
      }
      if (instring){
	if (s[i]=='"')
	  instring=false;
      }
      else {
	switch (s[i]){
	case '"':
	  instring=true;
	  break;
	case '(':
	  ++np;
	  break;
	case ')':
	  --np;
	  break;
	case '[':
	  ++nb;
	  break;
	case ']':
	  --nb;
	  break;
	}
      }
    }
    while (np<0 && i>=0 && s[i-1]==')'){
      --i;
      ++np;
    }
    while (nb<0 && i>=0 && s[i-1]==']'){
      --i;
      ++nb;
    }
    s=s.substr(0,i);
    if (nb<0)
      cerr << "Too many ]" << endl;
    if (np<0)
      cerr << "Too many )" << endl;
    if (nb>0)
      s=s+string(nb,']');
    if (np>0)
      s=s+string(np,')');
    BEGIN(INITIAL); 
    index_status=0;
    opened_quote=0;
    in_rpn=0;
    lexer_line_number=1;
    first_error_line=0;
    l=s.size();
    for (;l;l--){
      if (s[l-1]!=' ')
	break;
    }
    /* if (l && ( (s[l-1]==';') || (s[l-1]==':')))
       l--; */
    string ss;
    for (int i=0;i<l;++i){
      if (s[i]=='.'){
	if ( i && (i<l-1) && (s[i-1]!=' ') && (s[i+1]=='.') ){
	  ss+= " ..";
	  ++i;
	}
	else
	  ss+='.';
      }
      else {
	if (maple_mode > 0 && maple_mode !=3){
	  if (s[i]=='#')
	    ss += "//";
	  else
	    ss += s[i];
	}
	else
	  ss+=s[i];
      }
    }
    lexer_string = ss+"\n";
    curr_pos = 0;
  }
    
  // current lexer_string
  std::string the_lexer_string(){
    return lexer_string;
  }

  // current lexer_string
  int the_lexer_pos(){
    return curr_pos;
  }
    
  // Add to the list of predefined symbols
  void set_lexer_symbols(const vecteur & l){
    if (initialisation_done && (l==list_one_letter__IDNT) )
      return;
    initialisation_done=true;
    const_iterateur it=l.begin(),itend=l.end();
    for (; it!=itend; ++it) {
      if (it->type!=_IDNT)
	continue;
      sym_tab::const_iterator i = syms.find(* (it->_IDNTptr->name));
      if (i==syms.end())
	syms[* (it->_IDNTptr->name)] = *it;
    }
  }


#ifndef NO_NAMESPACE_GIAC
} // namespace giac
#endif // ndef NO_NAMESPACE_GIAC
